#   Meerkat: a product to add meerkat news to a site
#   Copyright (C) 2001  Carlos de la Guardia
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#    

import sys
import string
import time

import Globals
from Globals import Persistent, Acquisition
import AccessControl
import OFS

import ZODB, ZODB.FileStorage

import xmlrpclib

class Meerkat(
    OFS.SimpleItem.Item,
    Acquisition.Implicit,
    Persistent,
    AccessControl.Role.RoleManager,
    ):
    "Meerkat product"

    __ac_permissions__=(
        ('View management screens',('manage_main',)),
        ('View', ('index_html')),
        ('Change settings',('manage_edit',),('Manager',)),
        )

    manage_options=(
        {'label':'Properties','action':'manage_main'},
        {'label':'View','action':''},
        )+OFS.SimpleItem.SimpleItem.manage_options

    meta_type='Meerkat'

    index_html=Globals.HTMLFile("Meerkat", globals())
    manage_main=Globals.HTMLFile("edit_Meerkat",globals())

    def __init__(self, id, server):
        self.id=id
        self.server=server
	self.category=0
	self.num_items=5
	self.refresh=3600
	self._v_last_refresh=0
	self._v_categories=[]
	self._v_items={}
	self._v_channels=[]
	self._v_channelsbyc={}

    def manage_edit(self, server, category, num_items, refresh, REQUEST=None):
        "Edit the settings."
        self.server=server
	self.category=category
	self.num_items=num_items
	self.refresh=refresh
	self._v_last_refresh=0
        if REQUEST is not None:
            return Globals.MessageDialog(title='Edited',message='<strong>%s</strong> has been edited.' % self.id, action='manage_main',)

    def getNews(self):
	"Get configured news items"
	if self.category!='0':
	    category=self.category
	else:
	    category=None
	num_items=self.num_items
	items=self.getItems(category=category, num_items=num_items)
	return items
	
    def getRelatedNews(self, searchword):
	"Get related news items"
	num_items=self.num_items
	items=self.getItems(search=searchword, num_items=num_items)
	return items
	
    def getNewsHTML(self,style):
	"Get configured news items in HTML format"
	if self.category!='0':
	    category=self.category
	num_items=self.num_items
	items=self.getItems(category=category, num_items=num_items)
	html='<ul>'
	for item in items:
	    html=html+'<li><a href=\"'+item['link']+'\"> '+item['title']+'</a>'
	    if style=='1':
		html=html+'<br>'+item['description']
	html=html+'</ul>'
	return html
	
    def getRelatedNewsHTML(self,searchword,style):
	"Get related news items in HTML format"
	num_items=self.num_items
	items=self.getItems(search=searchword, num_items=num_items)
	html='<ul>'
	for item in items:
	    html=html+'<li><a href=\"'+item['link']+'\"> '+item['title']+'</a>'
	    if style=='1':
		html=html+'<br>'+item['description']
	html=html+'</ul>'
	return html
	
    def getRelatedNewsJS(self,searchword,style):
	"Get related news items in JS format"
	num_items=self.num_items
	items=self.getItems(search=searchword, num_items=num_items)
	html='Content-Type: text/plain\n\ndocument.write("<ul>");\n'
	for item in items:
	    item['title']=string.replace(item['title'],'\'','')
	    item['description']=string.replace(item['description'],'\'','')
	    html=html+'document.write(\'<li><a href=\"'+item['link']+'\"> '+item['title']+'</a>\');\n'
	    if style=='1':
		html=html+'document.write(\'<br>'+item['description']+'\');\n'
	html=html+'document.write(\"</ul>\");\n'
	return html
	
    def getNewsJS(self,style):
	"Get configured news items in JavaScript format"
	if self.category!='0':
	    category=self.category
	num_items=self.num_items
	items=self.getItems(category=category, num_items=num_items)
	html='Content-Type: text/plain\n\ndocument.write("<ul>");\n'
	for item in items:
	    item['title']=string.replace(item['title'],'\'','')
	    item['description']=string.replace(item['description'],'\'','')
	    html=html+'document.write(\'<li><a href=\"'+item['link']+'\"> '+item['title']+'</a>\');\n'
	    if style=='1':
		html=html+'document.write(\'<br>'+item['description']+'\');\n'
	html=html+'document.write(\"</ul>\");\n'
	return html
	
    def getCategories(self):
	"Get all Meerkat categories"
	server=xmlrpclib.Server(self.server)
	if self._checkTime():
		categories=tuple(server.meerkat.getCategories())
		self._v_categories=categories
	else:
		categories=self._v_categories
	return categories

    def getChannels(self):
	"Get all Meerkat channels"
	server=xmlrpclib.Server(self.server)
	if self._checkTime():
		channels=tuple(server.meerkat.getChannels())
		self._v_channels=channels
	else:
		channels=self._v_channels
	return channels

    def getChannelsByCategory(self, category):
	"Get all Meerkat channels for a given category"
	server=xmlrpclib.Server(self.server)
	t=self._checkTime()
	if self._v_channelsbyc.has_key(category):
		channels=self._v_channelsbyc[category]
	else:
		channels=tuple(server.meerkat.getChannelsByCategory(category))
		self._v_channelsbyc[category]=channels
	return channels

    def getItems(self, channel=0, category=0, item='', search='', time_period='', profile=0, mob=0, url='', ids=0, descriptions=0, categories=0, channels=0, dates=0, num_items=''):
	"Get items using all parameters"
	server=xmlrpclib.Server(self.server)
	query={}
	if channel:
	    query['channel']=channel
	if category:
	    query['category']=category
	if item:
	    query['item']=item
	if search:
	    query['search']=search
	if time_period:
	    query['time_period']=time_period
	if profile:
	    query['profile']=profile
	if mob:
	    query['mob']=mob
	if url:
	    query['url']=url
	if ids:
	    query['ids']=ids
	if descriptions:
	    query['descriptions']=descriptions
	if categories:
	    query['categories']=categories
	if channels:
	    query['channels']=channels
	if dates:
	    query['dates']=dates
	if num_items:
	    query['num_items']=num_items
	t=self._checkTime()
	querykey=tuple(query.values())
	if self._v_items.has_key(querykey):
		items=self._v_items[querykey]
	else:
		items=tuple(server.meerkat.getItems(query))
		self._v_items[querykey]=items
	return items

    def _checkTime(self):
	if not hasattr(self,'_v_last_refresh'):
		self._v_last_refresh=0
	timegone=0
	if time.time()-self.refresh>self._v_last_refresh:
		self._v_last_refresh=time.time()
		self._v_items={}
		self._v_channels=[]
		self._v_categories=[]
		self._v_channelsbyc={}
		timegone=1
	return timegone
	

Globals.default__class_init__(Meerkat)

def manage_addMeerkat(self, id, server, REQUEST=None):
    "Create a Meerkat"
    meerkat=Meerkat(id, server)
    self._setObject(id, meerkat)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

manage_addMeerkatForm=Globals.HTMLFile('add_Meerkat', globals())
